#!/usr/bin/env python

# Example, standalone implementation of Neighborhood Correlation
# (see http://www.neighborhoodcorrelation.org)

# (C) 2008 Jacob Joseph <jmjoseph@andrew.cmu.edu>,
#          Nan Song, and Carnegie Mellon University
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import getopt, sys
from nc_base import nc_base

class nc_standalone( nc_base):
    """Neighborhood Correlation calculation using pure Python from BLAST scores"""

    def __init__(self, nc_thresh, smin_factor,
                 flatfile_name, num_residues):
        nc_base.__init__( self)
        
        self.new_run( 0, None, nc_thresh, smin_factor,
                      0, num_residues)

        self.fetch_score_dict_flatfile( flatfile_name)


def usage():
    s = """Neighborhood Correlation (standalone)

    This implementation is meant as a practical implementation, and
    has no dependencies beyond a basic Python installation.  It has
    been tested on a Linux computer, but may function on MacOS or
    Windows.

    For details of Neighborhood Correlation please refer to
    http://www.neighborhoodcorrelation.org/ or the publication:

    Sequence Similarity Network Reveals Common Ancestry of Multidomain
    Proteins
    Song N, Joseph JM, Davis GB, Durand D
    PLoS Computational Biology 4(5): e1000063
    doi:10.1371/journal.pcbi.1000063

    (C) 2008 Jacob Joseph <jmjoseph@andrew.cmu.edu>,
    Nan Song, and Carnegie Mellon University
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 3
    of the License, or (at your option) any later version.

Usage:

    'nc_standalone.py -f <flatfile> [options]'

    Output will be printed to stdout in the same three-column format
    as used for input.

Options:

     -f, --flatfile <filename>   (required)
          A white-space delimited file of BIT-scores from BLAST.  Three
          columns are expected, of the format:

          --------------------------------------
          seq_id_0   seq_id_1   bit_score
          seq_id_2   seq_id_3   bit_score
          ....
          --------------------------------------
          
          No column heading should be provided.  Please refer to
          http://www.ncbi.nlm.nih.gov/books/bv.fcgi?rid=handbook.section.614
          for an explanation of BIT-score.

     -o, --output <filename>
          Write score output to a file.  If omitted, the score list is
          printed to stdout.
     
     -h, --help
          Print this help message.

     --num_residues <integer>
          Number of residues in the sequence database.  Used to
          calculate SMIN, the lowest expected bit_score.  If
          unspecified, an estimate of 537 residues per sequence will
          be used, which is an average of mouse and human SwissProt
          sequences.

     --nc_thresh <0.05>
          NC reporting threshold.  Calculated values below this
          threshold will not be reported.  Conservatively defaults to
          0.05.

     --smin_factor <0.95>
          SMIN factor.  Calculate SMIN from the expected random
          BIT-score scaled by this factor. Defaults to 0.95.
          """
    return s


def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:o:",
                                   ["help", "nc_thresh=", "smin_factor=",
                                    "flatfile=", "output=",
                                    "num_residues="])
        
    except getopt.GetoptError:
        # print help information and exit:
        print usage()
        raise

    help = False
    nc_thresh = 0.05
    smin_factor = 0.95
    score_type = 'bit_score'
    flatfile_name = None
    output_name = None
    num_residues = None
    for o, a in opts:
        if o in ("-h", "--help"):
            help=True
        if o == '--nc_thresh': nc_thresh = float(a)
        if o == '--smin_factor' : smin_factor = float(a)
        if o in ( '-f', '--flatfile'): flatfile_name = a
        if o in ( '-o', '--output'): output_name = a
        if o == "--num_residues": num_residues = int(a)
        
    if help or flatfile_name is None:
        print usage()
        sys.exit()

    ncrun = nc_standalone(nc_thresh, smin_factor,
                         flatfile_name, num_residues)

    if output_name is not None:
        out_fd = open(output_name, 'w')
    else:
        out_fd = sys.stdout
        
    ncrun.calculate_nc_queries( out_fd=out_fd)

    if out_fd != sys.stdout: out_fd.close()

    return ncrun

if __name__ == "__main__":
    ncrun = main()

